//
//  CustomTextMessageSizeCalculator.swift
//  ChatExample
//
//  Created by Vignesh J on 30/04/21.
//  Copyright © 2021 MessageKit. All rights reserved.
//

import MessageKit
import UIKit

class TSTextLayoutSizeCalculator: TSLayoutSizeCalculator {
    
    
    //MARK: ###################################消息label###################################
    //消息容器的 UIEdgeInsets
    static var cellMessagelabelEdge:UIEdgeInsets = UIEdgeInsets(top: 12, left: 12, bottom: 12, right: 12)
    static var messageLabelFont = UIFont.font(size: 16.0)
    static var cellMessagelabelMinSize:CGSize = CGSizeMake(24, 24)

    var cellMessagelabelLeadingMaxWidth:CGFloat{
        messageContainerLeadingMaxWidth - Self.cellMessagelabelEdge.left - Self.cellMessagelabelEdge.right
    }
    
    var cellMessagelabelTraingMaxWidth:CGFloat{
        messageContainerTraingMaxWidth - Self.cellMessagelabelEdge.left - Self.cellMessagelabelEdge.right
    }
    

    func cellMessagelabelMaxWidth(fromCurrentSender: Bool) -> CGFloat {
        let maxWidth = fromCurrentSender ? cellMessagelabelTraingMaxWidth: cellMessagelabelLeadingMaxWidth
        return maxWidth
    }
    
    //消息容器的高度
    override func cellContentHeight(for message: MessageType,at indexPath: IndexPath, fromCurrentSender: Bool)-> CGFloat {
        let superH = super.cellContentHeight(for: message, at: indexPath, fromCurrentSender: fromCurrentSender)
        
        var height = superH
        let size = messageLabelSize(for: message, at: indexPath, fromCurrentSender: fromCurrentSender)
        let messagecontainerH = size.height + Self.cellMessagelabelEdge.top + Self.cellMessagelabelEdge.bottom
        
        height = height + messagecontainerH
        
        let minHeight = Self.cellMessageContainerMinSize.height
        if height < minHeight {
            height = minHeight
        }
        
        return height
    }
    
    func messageLabelSize(for message: MessageType,at _: IndexPath, fromCurrentSender: Bool) -> CGSize{
        let attributedText: NSAttributedString
//        var sendText = ""
        let textMessageKind = message.kind
        switch textMessageKind {
        case .attributedText(let text):
            attributedText = text
//            sendText = text.string
        case .text(let text), .emoji(let text):
            attributedText = NSAttributedString(string: text, attributes: [.font: Self.messageLabelFont])
//            sendText = text
        default:
            fatalError("messageLabelSize received unhandled MessageDataType: \(message.kind)")
        }
        let maxWidth = cellMessagelabelMaxWidth(fromCurrentSender: fromCurrentSender)
//        let textSize = sendText.height(ofFont: Self.messageLabelFont, maxWidth: maxWidth)

        //用 label 计算更加精准
        var size = UILabel.getAttributedTextSize(attributedText: attributedText, maxWidth: maxWidth)
//        debugPrint("attributedText size=\(size)")
//        var size1 = attributedText.size(consideringWidth: maxWidth)

        let minSize = Self.cellMessagelabelMinSize
        if size.width < minSize.width {
            size.width = minSize.width
        }
        if size.height < minSize.height {
            size.height = minSize.height
        }
        return size
    }
}
