//
//  TSCommonloadingView.swift
//  AIEmoji
//
//  Created by 100Years on 2025/1/17.
//

import Kingfisher

class TSCommonloadingView: TSBaseView {
    
    lazy var imageView: UIImageView = {
        let imageView = UIImageView.createImageView(imageName: "failed")
        imageView.isHidden = true
        return imageView
    }()
    
    lazy var animatedImageView: AnimatedImageView = {
        let animatedImageView = AnimatedImageView()
        animatedImageView.autoPlayAnimatedImage = false
        if let gifURL = Bundle.main.url(forResource: "rotatingAnimation", withExtension: "gif") {
            animatedImageView.kf.setImage(with: gifURL, options: [.cacheOriginalImage]) { result in
                switch result {
                case .success(let value):
                    print("GIF 加载成功: \(value.source.url?.absoluteString ?? "")")
                case .failure(let error):
                    print("GIF 加载失败: \(error.localizedDescription)")
                }
            }
        }
        
        return animatedImageView
    }()
    
    lazy var textLabel: UILabel = {
        let textLabel = UILabel.createLabel(font: .font(size: 12),textColor: .fromHex("#FFFFFF",alpha: 0.6))
        return textLabel
    }()
    
    private var targetView: UIView = UIView()
    
    override func creatUI() {
        contentView.addSubview(imageView)
        imageView.snp.makeConstraints { make in
            make.width.height.equalTo(200.0)
            make.centerX.equalToSuperview()
            make.top.equalToSuperview()
        }
        
        contentView.addSubview(animatedImageView)
        animatedImageView.snp.makeConstraints { make in
            make.width.height.equalTo(200.0)
            make.centerX.equalToSuperview()
            make.top.equalToSuperview()
        }
  
        contentView.addSubview(textLabel)
        textLabel.snp.makeConstraints { make in
            make.height.equalTo(18.0)
            make.top.equalTo(imageView.snp.bottom).offset(16)
            make.leading.bottom.trailing.equalToSuperview()
        }
    }
    
    
    override func dealThings() {
        // 监听应用生命周期事件
        NotificationCenter.default.addObserver(
            self,
            selector: #selector(handleAppDidEnterBackground),
            name: UIApplication.didEnterBackgroundNotification,
            object: nil
        )
        
        NotificationCenter.default.addObserver(
            self,
            selector: #selector(handleAppWillEnterForeground),
            name: UIApplication.willEnterForegroundNotification,
            object: nil
        )
    }
    
    
    lazy var isRotating = false{
        didSet{
            if isRotating == true{
                startRotating(view: imageView)
            }else{
                stopRotating(view: imageView)
            }
        }
    }
    
    
    /// 开始旋转
    func startRotating(view:UIView,duration: Double = 2.0) {
        animatedImageView.startAnimating()
    }

    /// 停止旋转
    func stopRotating(view:UIView) {
        animatedImageView.stopAnimating()
    }
    
    func showLoading(text:String){
        animatedImageView.isHidden = false
        imageView.isHidden = true
        
        textLabel.text = text
        isRotating = true
    }
    
    func showError(text:String){
        animatedImageView.isHidden = true
        
        imageView.isHidden = false
        imageView.image = UIImage(named: "failed")
        
        textLabel.text = text
        isRotating = false
    }
    
//    /// 开始旋转
//    func startRotating(view:UIView,duration: Double = 2.0) {
//        targetView = view
//        stopRotating(view: view)
//        let rotationAnimation = CABasicAnimation(keyPath: "transform.rotation.z")
//        rotationAnimation.toValue = CGFloat.pi * 2
//        rotationAnimation.duration = duration
//        rotationAnimation.isCumulative = true
//        rotationAnimation.repeatCount = .infinity
//
//        view.layer.add(rotationAnimation, forKey: "rotationAnimation")
//    }
//
//    /// 停止旋转
//    func stopRotating(view:UIView) {
//        view.layer.removeAnimation(forKey: "rotationAnimation")
//    }
//    
//    func showLoading(text:String){
//        imageView.image = UIImage(named: "rotating")
//        textLabel.text = text
//        isRotating = true
//    }
//    
//    func showError(text:String){
//        imageView.image = UIImage(named: "failed")
//        textLabel.text = text
//        isRotating = false
//    }

    
    
    @objc private func handleAppDidEnterBackground() {
        stopRotating(view: targetView)
    }

    @objc private func handleAppWillEnterForeground() {
        isRotating = isRotating
    }
}
